@@ -1,11 +1,11 @@
 ;;; osm.el --- OpenStreetMap viewer -*- lexical-binding: t -*-
 
-;; Copyright (C) 2022-2024 Free Software Foundation, Inc.
+;; Copyright (C) 2022-2025 Free Software Foundation, Inc.
 
 ;; Author: Daniel Mendler <mail@daniel-mendler.de>
 ;; Maintainer: Daniel Mendler <mail@daniel-mendler.de>
 ;; Created: 2022
-;; Version: 1.5
+;; Version: 1.7
 ;; Package-Requires: ((emacs "28.1") (compat "30"))
 ;; URL: https://github.com/minad/osm
 ;; Keywords: network, multimedia, hypermedia, mouse
@@ -177,9 +177,10 @@ the domain name and the :user to the string \"apikey\"."
 (defcustom osm-pin-colors
   '((osm-selected . "#e20")
     (osm-bookmark . "#f80")
-    (osm-poi . "#88f")
     (osm-home . "#80f")
-    (osm-track . "#00a"))
+    (osm-track . "#00e")
+    (osm-gpx-poi . "#88f")
+    (osm-gpx-track . "#88f"))
   "Colors of pins."
   :type '(alist :key-type symbol :value-type string))
 
@@ -269,6 +270,7 @@ Should be at least 7 days according to the server usage policies."
 ;;;###autoload (autoload 'osm-prefix-map "osm" nil t 'keymap)
 (defalias 'osm-prefix-map osm-prefix-map)
 
+;; TODO add support for touchscreen-pinch for zooming.
 (defvar-keymap osm-mode-map
   :doc "Keymap used by `osm-mode'."
   :parent (make-composed-keymap osm-prefix-map special-mode-map)
@@ -654,7 +656,8 @@ Local per buffer since the overlays depend on the zoom level.")
                             (osm--update)))))))
     (setq track-mouse 'dragging)
     (set-transient-map map
-                       (lambda () (eq (car-safe last-input-event) 'mouse-movement))
+                       (lambda () (eq (car-safe last-command-event)
+                                      'mouse-movement))
                        (lambda () (setq track-mouse nil)))))
 
 (defun osm--zoom-in-wheel (_n)
@@ -889,14 +892,20 @@ Local per buffer since the overlays depend on the zoom level.")
               mwheel-scroll-down-function #'osm--zoom-in-wheel
               mwheel-scroll-left-function #'osm--zoom-out-wheel
               mwheel-scroll-right-function #'osm--zoom-in-wheel
-              bookmark-make-record-function #'osm--bookmark-record-default)
+              bookmark-make-record-function #'osm--bookmark-record-default
+              imenu-create-index-function #'osm--imenu-index)
   (when (boundp 'mwheel-coalesce-scroll-events)
     (setq-local mwheel-coalesce-scroll-events t))
   (when (boundp 'pixel-scroll-precision-mode)
     (setq-local pixel-scroll-precision-mode nil))
   (add-hook 'change-major-mode-hook #'osm--barf-change-mode nil 'local)
   (add-hook 'write-contents-functions #'osm--barf-write nil 'local)
-  (add-hook 'window-size-change-functions #'osm--resize nil 'local))
+  (add-hook 'window-size-change-functions
+            (let ((buf (current-buffer)))
+              (lambda (_)
+                (with-current-buffer buf
+                  (osm--update))))
+            nil 'local))
 
 (defun osm--barf-write ()
   "Barf for write operation."
@@ -925,8 +934,10 @@ Local per buffer since the overlays depend on the zoom level.")
            (pcase-let ((`(,lat ,lon ,zoom) (bookmark-prop-get bm 'coordinates)))
              (funcall fun 'osm-bookmark lat lon zoom (car bm))))
   (dolist (file osm--gpx-files)
+    (when-let ((start (caaadr file)))
+      (funcall fun 'osm-gpx-track (car start) (cdr start) 10 (car file)))
     (cl-loop for (lat lon name) in (cddr file) do
-             (funcall fun 'osm-poi lat lon 15 name)))
+             (funcall fun 'osm-gpx-poi lat lon 15 name)))
   (cl-loop for (lat lon name) in osm--track do
            (funcall fun 'osm-track lat lon 15 name)))
 
@@ -1141,11 +1152,6 @@ xmlns='http://www.w3.org/2000/svg' xmlns:xlink='http://www.w3.org/1999/xlink'>
     (setq osm--overlays nil)
     (osm--update)))
 
-(defun osm--resize (&rest _)
-  "Resize buffer."
-  (when (eq major-mode #'osm-mode)
-    (osm--update)))
-
 (defun osm--header-button (text action)
   "Format header line button with TEXT and ACTION."
   (propertize text
@@ -1587,6 +1593,20 @@ When called interactively, call the function `osm-home'."
       (`(,name ,_group ,id ,lat ,lon ,zoom) (osm--goto lat lon zoom nil id name))
       (_ (user-error "No pin selected")))))
 
+(defun osm--imenu-index ()
+  "Create Imenu index."
+  (let (index)
+    (osm--each-pin
+     (lambda (id lat lon zoom name)
+       (push (list name (vector lat lon zoom) #'osm--imenu-goto)
+             (alist-get (capitalize (substring (symbol-name id) 4))
+                        index nil nil #'equal))))
+    (sort index (lambda (x y) (string< (car x) (car y))))))
+
+(defun osm--imenu-goto (_name pos)
+  "Goto Imenu POS."
+  (osm--goto (aref pos 0) (aref pos 1) (aref pos 2) nil nil nil))
+
 (defun osm--fetch-json (url)
   "Get json from URL."
   (osm--check-libraries)
@@ -1646,12 +1666,12 @@ See `osm-search-server' and `osm-search-language' for customization."
            (assoc
             (minibuffer-with-setup-hook
                 (lambda ()
+                  ;; TODO Use `eager-display' on Emacs 31
                   (when (and (eq completing-read-function #'completing-read-default)
                              (not (bound-and-true-p vertico-mode))
                              (not (bound-and-true-p icomplete-mode)))
                     (let ((message-log-max nil)
                           (inhibit-message t))
-                      ;; Show matches immediately for default completion.
                       (minibuffer-completion-help))))
               (completing-read
                (format "Matches for '%s': " needle)
